@@ -18,7 +18,12 @@ use IO::Handle;
 
 $dbg  = 0;
 $prog = "Signify";
-$vers = "1.06";
+$vers = "1.07";
+srand();
+
+
+# If there is no $ENV{HOME}, set it from /etc/passwd.
+$ENV{HOME} = (getpwuid($<))[7] unless exists $ENV{HOME};
 
 
 %vars			= %ENV;
@@ -26,6 +31,7 @@ $vars{PROGRAM}	= "$prog v$vers";
 $vars{SIGWIDTH}	= 79;
 $vars{WEBSITE}	= "http://www.debian.org/";
 $vars{DOLSIGN}	= "\$";
+$vars{BLANK}	= "";
 $perlvarnam		= "_perl_";
 $perlvarnum		= 1;
 $CurFile		= "";
@@ -77,12 +83,12 @@ sub OpenFile {
 	}
 
 	if (grep(/^\Q$file\E$/,@OpenFiles)) {
-		die "Error: File '$file' is read recursively\n";
+		die "Signify: File '$file' is read recursively\n";
 	}
 
 	$CurFile = $file;
 	$CurPath = new IO::Handle;
-	open($CurPath,"<$file") || die "Error: Could not read file '$file' -- $!\n";
+	open($CurPath,"<$file") || die "Signify: Could not read file '$file' -- $!\n";
 	print STDERR "Starting file '$CurFile'\n" if $dbg;
 }
 
@@ -251,8 +257,7 @@ sub GenerateSig {
 					$status = $? >> 8;
 					$signal = $? & 255;
 					if ($?) {
-						print STDERR ("Shell command returned $status",
-									  " exit status",
+						print STDERR ("Shell command returned $status exit status",
 									  $signal ? " (killed on signal $signal)" : "",
 									  "\n") if $dbg;
 					} else {
@@ -399,8 +404,7 @@ sub GenerateSig {
 				$status = $? >> 8;
 				$signal = $? & 255;
 				if ($?) {
-					print STDERR ("Shell command returned $status",
-								  " exit status",
+					print STDERR ("Shell command returned $status exit status",
 								  $signal ? " (killed on signal $signal)" : "",
 								  "\n") if $dbg;
 				} else {
@@ -445,26 +449,31 @@ sub GenerateSig {
 sub MakeFifo {
 	my ($fifo) = @_;
 
-	die "Error: '$fifo' already exists and is not a FIFO\n" if (-e $fifo && ! -p $fifo);
+	$fifo =~ s!^~/!$ENV{HOME}/!;
+	die "Signify: '$fifo' already exists and is not a FIFO\n" if (-e $fifo && ! -p $fifo);
 	system("mkfifo -m 644 $fifo") if (! -p $fifo);
-	die "Error: Could not make FIFO '$fifo'\n" if (! -p $fifo);
+	die "Signify: Could not make FIFO '$fifo'\n" if (! -p $fifo);
 }
 
 
 
 sub LockFifo {
-	my($fifo) = @_;
-	my $lock = "$fifo.lock";
+	my($fifo,) = @_;
+	my $lock = "$fifo.lock-$hostnam";
+	my $pid;
 
-	if (open(LOCK,"<$lock")) {
-		my $pid = <LOCK>;
-		chomp $pid;
-		kill 1,$pid if ($pid > 1 && $pid != $$);
+	if (! -e $lock) {
+		open(LOCK,">>$lock") || die "Signify: Could not create file '$lock' -- $!\n";
 		close(LOCK);
 	}
 
-	open(LOCK,">$lock") || die "Error: Could not write file '$lock' -- $!\n";
-	print LOCK $$;
+	open(LOCK,"+<$lock") || die "Signify: Could not update file '$lock' -- $!\n";
+	flock(LOCK,2) || die "Signify: could not lock file '$lock' -- $!\n";
+	$pid = <LOCK>;
+	chomp $pid;
+	kill 15,$pid if ($pid > 1 && $pid != $$);
+	seek LOCK,0,0;
+	print LOCK $$,"\n";
 	close(LOCK);
 }
 
@@ -489,21 +498,22 @@ foreach (@ARGV) {
 		next;
 	}
 
-	die "Error: Unknown parameter '$_'\n";
+	die "Signify: Unknown parameter '$_'\n";
 }
 
 
 $sectref = ReadFile($InputFile);
 $nowtime = 840000000;
+$hostnam = $ENV{HOSTNAME};
+$hostnam = $ENV{HOST} unless $hostnam;
 
 
 if ($OutputFifo) {
 	local $SIG{PIPE} = sub {};
 	LockFifo($OutputFifo);
 	while (1) {
-		srand(time() ^ $$);
 		utime $nowtime,$nowtime,$OutputFifo;
-		open(FIFO,">$OutputFifo") || die "Error: Could not write to '$OutputFifo' -- $!\n";
+		open(FIFO,">$OutputFifo") || die "Signify: Could not write to '$OutputFifo' -- $!\n";
 		@lines = GenerateSig($sectref);
 		foreach (@lines) {
 			s/\s+$// unless m/^--/;
@@ -514,7 +524,6 @@ if ($OutputFifo) {
 		sleep(1);
 	}
 } else {
-	srand(time() ^ $$);
 	@lines = GenerateSig($sectref);
 	foreach (@lines) {
 		s/\s+$// unless m/^--/;
